@@ -6,6 +6,9 @@
    system-dependent definitions for coreutils
    Copyright (C) 1989-2013 Free Software Foundation, Inc.
 
+   Modifications for GNU Datamash are
+   Copyright (C) 2014-2015 Assaf Gordon <assafgordon@gmail.com>
+
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
@@ -152,10 +155,6 @@ select_plural (uintmax_t n)
 #define __STRX__(x) #x
 #define __STR__(x) __STRX__(x)
 #define __STR_LINE__ __STR__(__LINE__)
-#define internal_error(x) \
-  do { \
-    error (EXIT_FAILURE, 0, \
-        "internal error at " __FILE__ ":" __STR_LINE__ " (" #x ")" ); \
-  } while (0)
+#define internal_error(x) assert(!#x)
 
 #endif /* CALC_SYSTEM_H */
