@@ -1,6 +1,6 @@
 /* GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013-2014 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2015 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -56,6 +56,6 @@ init_blank_table (void)
 
 /* Force generation of these inline'd symbols, needed to avoid
    "undefined reference" when compiling with coverage instrumentation.
-   See: http://stackoverflow.com/questions/16245521/c99-inline-function-in-c-file/16245669#16245669 */
-void print_field_separator();
-void print_line_separator();
+   See: http://stackoverflow.com/a/16245669 */
+void print_field_separator ();
+void print_line_separator ();
