@@ -1,6 +1,6 @@
 /* GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013,2014 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2015 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -20,6 +20,7 @@
 
 /* Written by Assaf Gordon */
 #include <config.h>
+#include <assert.h>
 #include <ctype.h>
 #include <math.h>
 #include <stdio.h>
@@ -27,23 +28,26 @@
 #include <string.h>
 #include <stdbool.h>
 
+#include "system.h"
 #include "xalloc.h"
 #include "size_max.h"
 
 #include "utils.h"
 
-/* On some systems (e.g. Cygwin) nanl is not defined,
-   and gnulib does not yet provide a replacment
-   (though it does provide 'isnanl' replacement) */
-#ifndef nanl
-  #define nanl nan
-#endif
+bool _GL_ATTRIBUTE_PURE
+is_na (const char* value, const size_t len)
+{
+  assert (value != NULL); /* LCOV_EXCL_LINE */
+  return (len==2 && (strncasecmp (value,"NA",2)==0))
+          || (len==3 && (strncasecmp (value,"N/A",3)==0))
+          || (len==3 && (strncasecmp (value,"NAN",3)==0));
+}
 
 
 /* Compare two flowting-point variables, while avoiding '==' .
 see:
 http://www.gnu.org/software/libc/manual/html_node/Comparison-Functions.html */
-int
+int _GL_ATTRIBUTE_PURE
 cmp_long_double (const void *p1, const void *p2)
 {
   const long double *a = (const long double *)p1;
@@ -51,11 +55,11 @@ cmp_long_double (const void *p1, const void *p2)
   return ( *a > *b ) - (*a < *b);
 }
 
-long double
-median_value ( const long double * const values, size_t n )
+long double _GL_ATTRIBUTE_PURE
+median_value (const long double * const values, size_t n)
 {
 #if 0
-  return percentile_value(values, n, 2.0/4.0);
+  return percentile_value (values, n, 2.0/4.0);
 #else
   /* Equivalent to the above, but slightly faster */
   return (n&0x01)
@@ -64,18 +68,18 @@ median_value ( const long double * const values, size_t n )
 #endif
 }
 
-/* This implementation follows R's summary() and quantile(type=7) functions.
+/* This implementation follows R's summary () and quantile (type=7) functions.
    See discussion here:
    http://tolstoy.newcastle.edu.au/R/e17/help/att-1067/Quartiles_in_R.pdf */
-long double percentile_value ( const long double * const values,
-                               const size_t n, const double percentile )
+long double _GL_ATTRIBUTE_PURE
+percentile_value (const long double * const values,
+                  const size_t n, const double percentile)
 {
   const double h = ( (n-1) * percentile ) ;
-  const size_t fh = floor(h);
+  const size_t fh = floor (h);
 
   /* Error in the calling parameters, should not happen */
-  if (n==0 || percentile<0.0 || percentile>100.0)
-    return 0; /* LCOV_EXCL_LINE */
+  assert (n>0 && percentile>=0.0 && percentile<=100.0); /* LCOV_EXCL_LINE */
 
   if (n==1)
     return values[0];
@@ -86,21 +90,22 @@ long double percentile_value ( const long double * const values,
 
 /* Given a sorted array of doubles, return the MAD value
    (median absolute deviation), with scale constant 'scale' */
-long double mad_value ( const long double * const values, size_t n, double scale )
+long double _GL_ATTRIBUTE_PURE
+mad_value (const long double * const values, size_t n, double scale)
 {
-  const long double median = median_value(values,n);
-  long double *mads = xnmalloc(n,sizeof(long double));
+  const long double median = median_value (values,n);
+  long double *mads = xnmalloc (n,sizeof (long double));
   long double mad = 0 ;
   for (size_t i=0; i<n; ++i)
-    mads[i] = fabsl(median - values[i]);
-  qsortfl(mads,n);
-  mad = median_value(mads,n);
-  free(mads);
+    mads[i] = fabsl (median - values[i]);
+  qsortfl (mads,n);
+  mad = median_value (mads,n);
+  free (mads);
   return mad * scale;
 }
 
-long double
-arithmetic_mean_value ( const long double * const values, const size_t n)
+long double _GL_ATTRIBUTE_PURE
+arithmetic_mean_value (const long double * const values, const size_t n)
 {
   long double sum=0;
   long double mean;
@@ -110,17 +115,18 @@ arithmetic_mean_value ( const long double * const values, const size_t n)
   return mean;
 }
 
-long double
-variance_value ( const long double * const values, size_t n, int df )
+long double _GL_ATTRIBUTE_PURE
+variance_value (const long double * const values, size_t n, int df)
 {
   long double sum=0;
   long double mean;
   long double variance;
 
-  if ( df<0 || (size_t)df == n )
-    return nanl("");
+  assert (df>=0); /* LCOV_EXCL_LINE */
+  if ( (size_t)df == n )
+    return nanl ("");
 
-  mean = arithmetic_mean_value(values, n);
+  mean = arithmetic_mean_value (values, n);
 
   sum = 0 ;
   for (size_t i = 0; i < n; i++)
@@ -132,7 +138,7 @@ variance_value ( const long double * const values, size_t n, int df )
 }
 
 long double
-stdev_value ( const long double * const values, size_t n, int df )
+stdev_value (const long double * const values, size_t n, int df)
 {
   return sqrtl ( variance_value ( values, n, df ) );
 }
@@ -141,7 +147,8 @@ stdev_value ( const long double * const values, size_t n, int df )
  Given an array of doubles, return the skewness
  'df' is degrees-of-freedom. Use DF_POPULATION or DF_SAMPLE (see above).
  */
-long double skewness_value ( const long double * const values, size_t n, int df )
+long double
+skewness_value (const long double * const values, size_t n, int df)
 {
   long double moment2=0;
   long double moment3=0;
@@ -149,9 +156,9 @@ long double skewness_value ( const long double * const values, size_t n, int df
   long double skewness;
 
   if (n<=1)
-    return nanl("");
+    return nanl ("");
 
-  mean = arithmetic_mean_value(values, n);
+  mean = arithmetic_mean_value (values, n);
 
   for (size_t i = 0; i < n; i++)
     {
@@ -162,34 +169,36 @@ long double skewness_value ( const long double * const values, size_t n, int df
   moment2 /= n;
   moment3 /= n;
 
-  /* can't use 'powl(moment2,3.0/2.0)' - not all systems have powl */
-  skewness = moment3 / sqrtl(moment2*moment2*moment2);
+  /* can't use 'powl (moment2,3.0/2.0)' - not all systems have powl */
+  skewness = moment3 / sqrtl (moment2*moment2*moment2);
   if ( df == DF_SAMPLE )
     {
       if (n<=2)
-        return nanl("");
-      skewness = ( sqrtl(n*(n-1)) / (n-2) ) * skewness;
+        return nanl ("");
+      skewness = ( sqrtl (n*(n-1)) / (n-2) ) * skewness;
     }
 
   return skewness;
 }
 
 /* Standard error of skewness (SES), given the sample size 'n' */
-long double SES_value ( size_t n )
+long double
+SES_value (size_t n)
 {
   if (n<=2)
-    return nanl("");
-  return sqrtl( (long double)(6.0*n*(n-1)) /
-                  ((long double)(n-2)*(n+1)*(n+3)) );
+    return nanl ("");
+  return sqrtl ( (long double)(6.0*n*(n-1))
+                  / ((long double)(n-2)*(n+1)*(n+3)) );
 }
 
 /* Skewness Test statistics Z = ( sample skewness / SES ) */
-long double skewnessZ_value ( const long double * const values, size_t n)
+long double
+skewnessZ_value (const long double * const values, size_t n)
 {
   const long double skew = skewness_value (values,n,DF_SAMPLE);
   const long double SES = SES_value (n);
-  if (isnan(skew) || isnan(SES) )
-    return nanl("");
+  if (isnan (skew) || isnan (SES) )
+    return nanl ("");
   return skew/SES;
 }
 
@@ -198,7 +207,8 @@ long double skewnessZ_value ( const long double * const values, size_t n)
  Given an array of doubles, return the excess kurtosis
  'df' is degrees-of-freedom. Use DF_POPULATION or DF_SAMPLE (see above).
  */
-long double excess_kurtosis_value ( const long double * const values, size_t n, int df )
+long double _GL_ATTRIBUTE_PURE
+excess_kurtosis_value (const long double * const values, size_t n, int df)
 {
   long double moment2=0;
   long double moment4=0;
@@ -206,9 +216,9 @@ long double excess_kurtosis_value ( const long double * const values, size_t n,
   long double excess_kurtosis;
 
   if (n<=1)
-    return nanl("");
+    return nanl ("");
 
-  mean = arithmetic_mean_value(values, n);
+  mean = arithmetic_mean_value (values, n);
 
   for (size_t i = 0; i < n; i++)
     {
@@ -224,60 +234,65 @@ long double excess_kurtosis_value ( const long double * const values, size_t n,
   if ( df == DF_SAMPLE )
     {
       if (n<=3)
-        return nanl("");
-      excess_kurtosis = ( ((long double)n-1) / (((long double)n-2)*((long double)n-3)) ) *
-                        ( (n+1)*excess_kurtosis + 6 ) ;
+        return nanl ("");
+      excess_kurtosis = ( ((long double)n-1)
+                          / (((long double)n-2)*((long double)n-3)) ) *
+                          ( (n+1)*excess_kurtosis + 6 ) ;
     }
 
   return excess_kurtosis;
 }
 
 /* Standard error of kurtisos (SEK), given the sample size 'n' */
-long double SEK_value ( size_t n )
+long double
+SEK_value (size_t n)
 {
-  const long double ses = SES_value(n);
+  const long double ses = SES_value (n);
 
   if (n<=3)
-    return nanl("");
+    return nanl ("");
 
-   return 2 * ses * sqrtl( (long double)(n*n-1) /
-                            ((long double)((n-3)*(n+5)))  );
+   return 2 * ses * sqrtl ( (long double)(n*n-1)
+                            / ((long double)((n-3)*(n+5)))  );
 }
 
 /* Kurtosis Test statistics Z = ( sample kurtosis / SEK ) */
-long double kurtosisZ_value ( const long double * const values, size_t n)
+long double
+kurtosisZ_value (const long double * const values, size_t n)
 {
   const long double kurt = excess_kurtosis_value (values,n,DF_SAMPLE);
   const long double SEK = SEK_value (n);
-  if (isnan(kurt) || isnan(SEK) )
-    return nanl("");
+  if (isnan (kurt) || isnan (SEK) )
+    return nanl ("");
   return kurt/SEK;
 }
 
 /*
  Chi-Squared - Cumulative distribution function,
  for the special case of DF=2.
- Equivalent to the R function 'pchisq(x,df=2)'.
+ Equivalent to the R function 'pchisq (x,df=2)'.
 */
-long double pchisq_df2(long double x)
+long double
+pchisq_df2 (long double x)
 {
-  return 1.0 - expl(-x/2);
+  return 1.0 - expl (-x/2);
 }
 
 /*
  Given an array of doubles, return the p-Value
  Of the Jarque-Bera Test for normality
    http://en.wikipedia.org/wiki/Jarque%E2%80%93Bera_test
- Equivalent to R's "jarque.test()" function in the "moments" library.
+ Equivalent to R's "jarque.test ()" function in the "moments" library.
  */
-long double jarque_bera_pvalue ( const long double * const values, size_t n )
+long double
+jarque_bera_pvalue (const long double * const values, size_t n )
 {
-  const long double k = excess_kurtosis_value(values,n,DF_POPULATION);
-  const long double s = skewness_value(values,n,DF_POPULATION);
+  const long double k = excess_kurtosis_value (values,n,DF_POPULATION);
+  const long double s = skewness_value (values,n,DF_POPULATION);
   const long double jb = (long double)(n*(s*s + k*k/4))/6.0 ;
-  const long double pval = 1.0 - pchisq_df2(jb);
-  if (n<=1 || isnan(k) || isnan(s))
-    return nanl("");
+  const long double pval = 1.0 - pchisq_df2 (jb);
+  if (n<=1 || isnan (k) || isnan (s))
+    return nanl ("");
   return pval;
 }
 
@@ -286,20 +301,21 @@ long double jarque_bera_pvalue ( const long double * const values, size_t n )
  returns the p-value,
  where the null-hypothesis is normal distribution.
 */
-long double dagostino_pearson_omnibus_pvalue( const long double * const values, size_t n)
+long double
+dagostino_pearson_omnibus_pvalue (const long double * const values, size_t n)
 {
   const long double z_skew = skewnessZ_value (values, n);
   const long double z_kurt = kurtosisZ_value (values, n);
   const long double DP = z_skew*z_skew + z_kurt*z_kurt;
-  const long double pval = 1.0 - pchisq_df2( DP );
+  const long double pval = 1.0 - pchisq_df2 (DP);
 
-  if (isnan(z_skew) || isnan(z_kurt))
-    return nanl("");
+  if (isnan (z_skew) || isnan (z_kurt))
+    return nanl ("");
   return pval;
 }
 
 
-long double
+long double _GL_ATTRIBUTE_PURE
 mode_value ( const long double * const values, size_t n, enum MODETYPE type)
 {
   /* not ideal implementation but simple enough */
@@ -311,14 +327,13 @@ mode_value ( const long double * const values, size_t n, enum MODETYPE type)
 
   for (size_t i=1; i<n; i++)
     {
-      bool eq = (cmp_long_double(&values[i],&last_value)==0);
+      bool eq = (cmp_long_double (&values[i],&last_value)==0);
 
       if (eq)
         seq_size++;
 
       if ( ((type==MODE) && (seq_size > best_seq_size))
-           ||
-           ((type==ANTIMODE) && (seq_size < best_seq_size)))
+           || ((type==ANTIMODE) && (seq_size < best_seq_size)))
         {
           best_seq_size = seq_size;
           best_value = last_value;
@@ -332,26 +347,33 @@ mode_value ( const long double * const values, size_t n, enum MODETYPE type)
   return best_value;
 }
 
-int
-cmpstringp(const void *p1, const void *p2)
+int _GL_ATTRIBUTE_PURE
+cmpstringp (const void *p1, const void *p2)
 {
   /* The actual arguments to this function are "pointers to
-   * pointers to char", but strcmp(3) arguments are "pointers
+   * pointers to char", but strcmp (3) arguments are "pointers
    * to char", hence the following cast plus dereference */
-  return strcmp(* (char * const *) p1, * (char * const *) p2);
+  return strcmp (* (char * const *) p1, * (char * const *) p2);
 }
 
-int
-cmpstringp_nocase(const void *p1, const void *p2)
+int _GL_ATTRIBUTE_PURE
+cmpstringp_nocase (const void *p1, const void *p2)
 {
   /* The actual arguments to this function are "pointers to
-   * pointers to char", but strcmp(3) arguments are "pointers
+   * pointers to char", but strcmp (3) arguments are "pointers
    * to char", hence the following cast plus dereference */
-  return strcasecmp(* (char * const *) p1, * (char * const *) p2);
+  return strcasecmp (* (char * const *) p1, * (char * const *) p2);
 }
 
 /* Sorts (in-place) an array of long-doubles */
-void qsortfl(long double *values, size_t n)
+void qsortfl (long double *values, size_t n)
 {
   qsort (values, n, sizeof (long double), cmp_long_double);
 }
+
+bool _GL_ATTRIBUTE_PURE
+hash_compare_strings (void const *x, void const *y)
+{
+  assert (x != y); /* LCOV_EXCL_LINE */
+  return STREQ (x, y) ? true : false;
+}
