@@ -2,7 +2,7 @@
 =pod
   Unit Tests for GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013,2014 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2015 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -29,6 +29,7 @@ use warnings;
 use Coreutils;
 use CuSkip;
 use CuTmpdir qw(datamash);
+use MIME::Base64 ;
 
 (my $program_name = $0) =~ s|.*/||;
 my $prog_bin = 'datamash';
@@ -38,8 +39,7 @@ my $prog_bin = 'datamash';
 ##  the full path of the binary, if the binary is on the $PATH.
 ##  So we try to detect what is the actual returned value of the program
 ##  in case of an error.
-my $prog = `$prog_bin --foobar 2>&1 | head -n 1 | cut -f1 -d:`;
-chomp $prog if $prog;
+my $prog = `$prog_bin ---print-progname`;
 $prog = $prog_bin unless $prog;
 
 ## Portability hack
@@ -104,6 +104,12 @@ K 6
 P 2
 EOF
 
+my $in_g5=<<'EOF';
+A 1
+AA 2
+AAA 3
+EOF
+
 my $in_hdr1=<<'EOF';
 x y z
 A 1 10
@@ -166,6 +172,13 @@ B 1
 B 2
 EOF
 
+my $in_cnt_uniq2=<<'EOF';
+a B
+A b
+A B
+A C
+EOF
+
 # When using whitespace, the second column is 1,2,3.
 # When using Tab, the second column is 10,20,30.
 my $in_tab1=<<"EOF";
@@ -207,6 +220,20 @@ A x 5
 B Y 6
 EOF
 
+my $in_sort_quote1=<<"EOF";
+A'1
+B'2
+A'3
+B'4
+EOF
+
+my $in_sort_quote2=<<'EOF';
+A"1
+B"2
+A"3
+B"4
+EOF
+
 ## NUL as end-of-line character
 my $in_nul1="A 1\x00A 2\x00B 3\x00B 4\x00";
 
@@ -252,6 +279,60 @@ my $out_large_buffer_last =
 "A " . "FooBar" x 200 . "\n" .
 "B " . "FooBar" x 400 . "\n" ;
 
+# Input with variable (and large) number of fields
+my $in_wide1 =
+"A " x 10 . "x\n" .
+"A " x 21 . "x\n" .
+"A " x 99 . "x\n" .
+"A " x 1000 . "x\n" .
+"A " x 2 . "x\n" ;
+
+
+my $in_empty1=<<'EOF';
+A::2
+B::3
+B::4
+EOF
+
+my $in_dup1=<<'EOF';
+X:Y
+1:a
+2:b
+3:a
+2:c
+EOF
+
+my $in_hdr_only=<<'EOF';
+X:Y:Z
+EOF
+
+=pod
+  Example:
+  my $data = "a 1\nb 2\n";
+  my $out = transform_column($data, 2, \&md5_hex);
+  # out => md5_hex("1") . "\n" . md5_hex("2") . "\n" ;
+=cut
+sub transform_column($$$)
+{
+  my $input_text = shift;
+  my $input_column = shift;
+  my $function = shift;
+
+  return join "",
+		map { "$_\n" }
+		map { &$function($_->[ $input_column - 1 ]) }
+		map { [ split / / ] }
+		split("\n", $input_text);
+}
+
+sub single_line_base64($)
+{
+	my $in = shift;
+	return encode_base64($in,'');
+}
+
+my $out_g1_base64 = transform_column ($in_g1, 2, \&single_line_base64);
+my $out_g1_debase64 = transform_column ($out_g1_base64, 1, \&decode_base64);
 
 my @Tests =
 (
@@ -271,63 +352,88 @@ my @Tests =
   # on a different architecture, would printf(%Lg) print something else?
   # Use OUT_SUBST to trim output to 1.3 digits
   ['b14', 'mean 1',     {IN_PIPE=>$in1},  {OUT => "5.454\n"},
-	  {OUT_SUBST=>'s/^(\d\.\d{3}).*/\1/'}],
+      {OUT_SUBST=>'s/^(\d\.\d{3}).*/\1/'}],
   ['b15', 'pstdev 1',   {IN_PIPE=>$in1},  {OUT => "2.742\n"},
-	  {OUT_SUBST=>'s/^(\d\.\d{3}).*/\1/'}],
+      {OUT_SUBST=>'s/^(\d\.\d{3}).*/\1/'}],
   ['b16', 'sstdev 1',   {IN_PIPE=>$in1},  {OUT => "2.876\n"},
-	  {OUT_SUBST=>'s/^(\d\.\d{3}).*/\1/'}],
+      {OUT_SUBST=>'s/^(\d\.\d{3}).*/\1/'}],
   ['b17', 'pvar 1',     {IN_PIPE=>$in1},  {OUT => "7.520\n"},
-	  {OUT_SUBST=>'s/^(\d\.\d{3}).*/\1/'}],
+      {OUT_SUBST=>'s/^(\d\.\d{3}).*/\1/'}],
   ['b18', 'svar 1',     {IN_PIPE=>$in1},  {OUT => "8.272\n"},
-	  {OUT_SUBST=>'s/^(\d\.\d{3}).*/\1/'}],
+      {OUT_SUBST=>'s/^(\d\.\d{3}).*/\1/'}],
   ['b19', 'countunique 1', {IN_PIPE=>$in1}, {OUT => "10\n"}],
   ['b20', 'first 1',    {IN_PIPE=>$in1},  {OUT => "1\n"}],
   ['b21', 'last 1',     {IN_PIPE=>$in1},  {OUT => "10\n"}],
   # This test just ensures the 'rand' operation is functioning.
   # It does not verify randomness (see datamash-rand.sh test for that).
   ['b22', 'rand 1',     {IN_PIPE=>$in1},  {OUT => "\n"},
-	  {OUT_SUBST=>'s/[0-9]+//'}],
+      {OUT_SUBST=>'s/[0-9]+//'}],
 
 
 
   ## Some error checkings
   ['e1',  'sum',  {IN_PIPE=>""}, {EXIT=>1},
-	  {ERR=>"$prog: missing field number after operation 'sum'\n"}],
+      {ERR=>"$prog: missing field number after operation 'sum'\n"}],
   ['e2',  'foobar',  {IN_PIPE=>""}, {EXIT=>1},
-	  {ERR=>"$prog: invalid operation 'foobar'\n"}],
+      {ERR=>"$prog: invalid operation 'foobar'\n"}],
   ['e3',  '',  {IN_PIPE=>""}, {EXIT=>1},
-	  {ERR=>"$prog: missing operation specifiers\n" .
-		  "Try '$prog --help' for more information.\n"}],
+      {ERR=>"$prog: missing operation specifiers\n" .
+             "Try '$prog --help' for more information.\n"}],
   ['e4',  'sum 1' ,  {IN_PIPE=>"a\n"}, {EXIT=>1},
-	  {ERR=>"$prog: invalid numeric input in line 1 field 1: 'a'\n"}],
+      {ERR=>"$prog: invalid numeric value in line 1 field 1: 'a'\n"}],
   ['e5',  '-g 4, sum 1' ,  {IN_PIPE=>"a\n"}, {EXIT=>1},
-	  {ERR=>"$prog: invalid field value for grouping ''\n"}],
+      {ERR=>"$prog: invalid empty grouping parameter\n"}],
   ['e6',  '-g 4,x sum 1' ,  {IN_PIPE=>"a\n"}, {EXIT=>1},
-	  {ERR=>"$prog: invalid field value for grouping 'x'\n"}],
+      {ERR=>"$prog: -H or --header-in must be used with named columns\n"}],
   ['e7',  '-g ,x sum 1' ,  {IN_PIPE=>"a\n"}, {EXIT=>1},
-	  {ERR=>"$prog: invalid field value for grouping ',x'\n"}],
+      {ERR=>"$prog: invalid empty grouping parameter\n"}],
   ['e8',  '-g 1,0 sum 1' ,  {IN_PIPE=>"a\n"}, {EXIT=>1},
-	  {ERR=>"$prog: invalid field value (zero) for grouping\n"}],
+      {ERR=>"$prog: invalid grouping parameter '0'\n"}],
   ['e9',  '-g 1X0 sum 1' ,  {IN_PIPE=>"a\n"}, {EXIT=>1},
-	  {ERR=>"$prog: invalid grouping parameter 'X0'\n"}],
+      {ERR=>"$prog: -H or --header-in must be used with named columns\n"}],
   ['e10',  '-g 1 -t XX sum 1' ,  {IN_PIPE=>"a\n"}, {EXIT=>1},
-	  {ERR=>"$prog: the delimiter must be a single character\n"}],
+      {ERR=>"$prog: the delimiter must be a single character\n"}],
+  ['e10.1',  '-g 1 -t "" sum 1' ,  {IN_PIPE=>"a\n"}, {EXIT=>1},
+      {ERR=>"$prog: the delimiter must be a single character\n"}],
   ['e11',  '--foobar' ,  {IN_PIPE=>"a\n"}, {EXIT=>1},
-	  {ERR=>"$prog: unrecognized option foobar\n" .
+      {ERR=>"$prog: unrecognized option foobar\n" .
                 "Try '$prog --help' for more information.\n"},
           # This ERR_SUBST is needed because on some systems (e.g. OpenBSD),
-          # The error message from 'getopt_long' is slightly different than GNU libc's.
-          {ERR_SUBST=>'s/(unknown|unrecognized) option.*(foobar).*/unrecognized option $2/'}],
+          # The error message from 'getopt_long' is slightly
+          # different than GNU libc's.
+          {ERR_SUBST=>'s/(unknown|unrecognized) option.*(foobar).*' .
+                      '/unrecognized option $2/'}],
   ['e12',  '-t" " -H unique 4' ,  {IN_PIPE=>$in_hdr1}, {EXIT=>1},
-	  {ERR=>"$prog: not enough input fields (field 4 requested, input has only 3 fields)\n"}],
-  ['e13',  'sum 6' ,  {IN_PIPE=>$in_g3}, {EXIT=>1},
-	  {ERR=>"$prog: invalid numeric input in line 1 field 6: ''\n"}],
+      {ERR=>"$prog: invalid input: field 4 requested, " .
+            "line 1 has only 3 fields\n"}],
+  ['e13',  '-t" " sum 6' ,  {IN_PIPE=>$in_g3}, {EXIT=>1},
+      {ERR=>"$prog: invalid input: field 6 requested, " .
+            "line 1 has only 3 fields\n"}],
   ['e14',  '--header-in -t: sum 6' ,  {IN_PIPE=>$in_hdr2}, {EXIT=>1},
-	  {ERR=>"$prog: invalid numeric input in line 2 field 6: ''\n"}],
+      {ERR=>"$prog: invalid input: field 6 requested, " .
+            "line 2 has only 3 fields\n"}],
   ['e15',  'sum foo' ,  {IN_PIPE=>"a"}, {EXIT=>1},
-	  {ERR=>"$prog: invalid column 'foo' for operation 'sum'\n"}],
+      {ERR=>"$prog: -H or --header-in must be used with named columns\n"}],
   ['e16',  '-t" " sum 2' ,  {IN_PIPE=>$in_invalid_num1}, {EXIT=>1},
-	  {ERR=>"$prog: invalid numeric input in line 3 field 2: '3a'\n"}],
+      {ERR=>"$prog: invalid numeric value in line 3 field 2: '3a'\n"}],
+  ['e17',  'sum 1' ,  {IN_PIPE=>"1e-20000\n"}, {EXIT=>1},
+      {ERR=>"$prog: invalid numeric value in line 1 field 1: '1e-20000'\n"}],
+  ['e18',  'sum 0' ,  {IN_PIPE=>"a"}, {EXIT=>1},
+      {ERR=>"$prog: invalid column '0' for operation 'sum'\n"}],
+  ['e19',  '-- sum -2' ,  {IN_PIPE=>"a"}, {EXIT=>1},
+      {ERR=>"$prog: invalid column '-2' for operation 'sum'\n"}],
+  ['e21',  'sum ""' ,  {IN_PIPE=>"a"}, {EXIT=>1},
+      {ERR=>"$prog: invalid empty column for operation 'sum'\n"}],
+  ['e22',  '-t" " -g 7 unique 1' ,  {IN_PIPE=>$in_hdr1}, {EXIT=>1},
+      {ERR=>"$prog: invalid input: field 7 requested, " .
+            "line 2 has only 3 fields\n"}],
+  ['e23',  '-t" " -g -2 unique 1' ,  {IN_PIPE=>$in_hdr1}, {EXIT=>1},
+      {ERR=>"$prog: invalid grouping parameter '-2'\n"}],
+  ['e24',  '-t" " --header-out -g 5 count 1', {IN_PIPE=>$in_hdr1}, {EXIT=>1},
+      {ERR=>"$prog: invalid input: field 5 requested, " .
+            "line 1 has only 3 fields\n"}],
+  ['e25',  '-g 1,,2 sum 1' ,  {IN_PIPE=>"a\n"}, {EXIT=>1},
+      {ERR=>"$prog: invalid empty grouping parameter\n"}],
 
   # No newline at the end of the lines
   ['nl1', 'sum 1', {IN_PIPE=>"99"}, {OUT=>"99\n"}],
@@ -340,11 +446,12 @@ my @Tests =
   [ 'emp4', '--header-out count 2', {IN_PIPE=>""},{OUT=>""}],
   [ 'emp5', '--full --header-in count 2', {IN_PIPE=>""},{OUT=>""}],
   [ 'emp6', '--full --header-out count 2', {IN_PIPE=>""},{OUT=>""}],
-  [ 'emp7', '--full --header-in --header-out count 2', {IN_PIPE=>""},{OUT=>""}],
-  [ 'emp8', '-g3,4 --full --header-in --header-out count 2', {IN_PIPE=>""},{OUT=>""}],
+  [ 'emp7', '--full --header-in --header-out count 2',
+     {IN_PIPE=>""},{OUT=>""}],
+  [ 'emp8', '-g3,4 --full --header-in --header-out count 2',
+     {IN_PIPE=>""},{OUT=>""}],
   [ 'emp9', '-g3 count 2', {IN_PIPE=>""},{OUT=>""}],
 
-
   ## Field extraction
   ['f1', '-W sum 1', {IN_PIPE=>$in2}, {OUT=>"5\n"}],
   ['f2', '-W sum 2', {IN_PIPE=>$in2}, {OUT=>"7\n"}],
@@ -366,6 +473,18 @@ my @Tests =
   ['f16', '-W collapse 2', {IN_PIPE=>$in3}, {OUT=>"2,5\n"}],
   ['f17', '-W collapse 3', {IN_PIPE=>$in3}, {OUT=>"3,6\n"}],
 
+  # Empty fields (2 is the empty field in 'in_empty1')
+  ['f18', '-t: -g1 count 3', {IN_PIPE=>$in_empty1}, {OUT=>"A:1\nB:2\n"}],
+  # String operations on an empty field should work
+  ['f19', '-t: -g1 count 2',       {IN_PIPE=>$in_empty1}, {OUT=>"A:1\nB:2\n"}],
+  ['f20', '-t: -g1 countunique 2', {IN_PIPE=>$in_empty1}, {OUT=>"A:1\nB:1\n"}],
+  #TODO: should collapsing two empty string result in a single comma?
+  ['f21', '-t: -g1 collapse 2',    {IN_PIPE=>$in_empty1}, {OUT=>"A:\nB:,\n"}],
+  ['f22', '-t: -g1 unique 2',      {IN_PIPE=>$in_empty1}, {OUT=>"A:\nB:\n"}],
+  # Numeric operation on an empty field should not work
+  ['f23', '-t: -g1 sum 2', {IN_PIPE=>$in_empty1}, {EXIT=>1},
+    {ERR=>"$prog: invalid numeric value in line 1 field 2: ''\n"}],
+
 
   # Test Absolute min/max
   ['mm1', 'min 1', {IN_PIPE=>$in_minmax}, {OUT=>"-700\n"}],
@@ -380,7 +499,13 @@ my @Tests =
   # Single group (key in column 1)
   ['g1.1', '-t" " -g1 sum 2',    {IN_PIPE=>$in_g1}, {OUT=>"A 195\n"}],
   ['g2.1', '-t" " -g1 median 2', {IN_PIPE=>$in_g1}, {OUT=>"A 42.5\n"}],
-  ['g3.1', '-t" " -g1 collapse 2', {IN_PIPE=>$in_g1}, {OUT=>"A 100,10,50,35\n"}],
+  ['g3.1', '-t" " -g1 collapse 2', {IN_PIPE=>$in_g1},
+     {OUT=>"A 100,10,50,35\n"}],
+  ['g4.1', '-t" " -g1 count 2',    {IN_PIPE=>$in_g5},
+     {OUT=>"A 1\nAA 1\nAAA 1\n"}],
+  # Group on the last column
+  ['grp5', '-t" " -g3 count 1',    {IN_PIPE=>$in_hdr1},
+     {OUT=>"z 1\n10 6\n20 1\n30 1\n11 1\n22 1\n33 1\n44 1\n"}],
 
   # 3 groups, single line per group, custom delimiter
   ['g7.1', '-g2 -t= mode 1', {IN_PIPE=>"1=A\n2=B\n3=C\n"},
@@ -411,7 +536,8 @@ my @Tests =
      {OUT=>"GroupBy(x) count(y)\nA 5\nB 3\nC 4\n"}],
 
   # Input and output header, with full line
-  ['hdr3', '-t" " -g 1 --full --header-in --header-out count 2',{IN_PIPE=>$in_hdr1},
+  ['hdr3', '-t" " -g 1 --full --header-in --header-out count 2',
+     {IN_PIPE=>$in_hdr1},
      {OUT=>"x y z count(y)\nA 1 10 5\nB 5 10 3\nC 8 11 4\n"}],
 
   # Output Header
@@ -420,7 +546,8 @@ my @Tests =
 
   # Output Header with --full
   ['hdr5', '-t" " -g 1 --full --header-out count 2', {IN_PIPE=>$in_g3},
-     {OUT=>"field-1 field-2 field-3 count(field-2)\nA 3 W 5\nB 17 Y 2\nC 23 Z 1\n"}],
+     {OUT=>"field-1 field-2 field-3 count(field-2)\n" .
+           "A 3 W 5\nB 17 Y 2\nC 23 Z 1\n"}],
 
   # Header without grouping
   ['hdr6', '-t" " --header-out count 2', {IN_PIPE=>$in_g3},
@@ -428,7 +555,8 @@ my @Tests =
 
   # Output Header, multiple ops
   ['hdr7', '-t" " -g 1 --header-out count 2 unique 3', {IN_PIPE=>$in_g3},
-     {OUT=>"GroupBy(field-1) count(field-2) unique(field-3)\nA 5 W,X\nB 2 Y,Z\nC 1 Z\n"}],
+     {OUT=>"GroupBy(field-1) count(field-2) unique(field-3)\n" .
+           "A 5 W,X\nB 2 Y,Z\nC 1 Z\n"}],
 
   # Headers, non white-space separator
   ['hdr8', '-g 1 -H -t: count 2 unique 3', {IN_PIPE=>$in_hdr2},
@@ -436,12 +564,46 @@ my @Tests =
 
   # Headers, non white-space separator, 3 operations
   ['hdr9', '-g 1 -H -t: count 2 unique 3 sum 2', {IN_PIPE=>$in_hdr2},
-     {OUT=>"GroupBy(x):count(y):unique(z):sum(y)\nA:5:W,X:39\nB:2:Y,Z:36\nC:1:Z:23\n"}],
+     {OUT=>"GroupBy(x):count(y):unique(z):sum(y)\n" .
+           "A:5:W,X:39\nB:2:Y,Z:36\nC:1:Z:23\n"}],
 
   # Headers, white-space separator, 3 operations
   ['hdr10', '-W -g 1 --header-in --header-out count 2',{IN_PIPE=>$in_hdr3},
      {OUT=>"GroupBy(x)\tcount(y)\nA\t5\nB\t3\nC\t4\n"}],
 
+  # Group + output headers on the last column
+  ['hdr11', '-t" " --header-out -g3 count 1',    {IN_PIPE=>$in_hdr1},
+     {OUT=>"GroupBy(field-3) count(field-1)\n" .
+           "z 1\n10 6\n20 1\n30 1\n11 1\n22 1\n33 1\n44 1\n"}],
+  # Group + input headers on last column
+  ['hdr12', '-t" " --header-in -g3 count 1',    {IN_PIPE=>$in_hdr1},
+     {OUT=>"10 6\n20 1\n30 1\n11 1\n22 1\n33 1\n44 1\n"}],
+  # Group + input/output headers on last column
+  ['hdr13', '-t" " -H -g3 count 1',    {IN_PIPE=>$in_hdr1},
+     {OUT=>"GroupBy(z) count(x)\n10 6\n20 1\n30 1\n11 1\n22 1\n33 1\n44 1\n"}],
+  # Input has only one header line (no data lines), and the user requested
+  # header-in and header-out => header line should be printed
+  ['hdr14', '-t: -H sum 1', {IN_PIPE=>$in_hdr_only},
+     {OUT=>"sum(X)\n"}],
+  ['hdr15', '-t: --full -H sum 1', {IN_PIPE=>$in_hdr_only},
+     {OUT=>"X:Y:Z:sum(X)\n"}],
+  ['hdr16', '-t: -s -g1 -H sum 2', {IN_PIPE=>$in_hdr_only},
+     {OUT=>"GroupBy(X):sum(Y)\n"}],
+  ['hdr17', '-t: --full -s -g1 -H sum 2', {IN_PIPE=>$in_hdr_only},
+     {OUT=>"X:Y:Z:sum(Y)\n"}],
+  ['hdr18', '-t: --header-in sum 1', {IN_PIPE=>$in_hdr_only},
+     {OUT=>""}],
+  ['hdr19', '-t: -H reverse', {IN_PIPE=>$in_hdr_only},
+     {OUT=>"Z:Y:X\n"}],
+  ['hdr20', '-t: --header-in reverse', {IN_PIPE=>$in_hdr_only},
+     {OUT=>""}],
+  ['hdr21', '-t: -H rmdup 1', {IN_PIPE=>$in_hdr_only},
+     {OUT=>$in_hdr_only}],
+  ['hdr22', '-t: --header-in rmdup 1', {IN_PIPE=>$in_hdr_only},
+     {OUT=>""}],
+  ['hdr23', '-t: --header-in rmdup 1', {IN_PIPE=>""}, {OUT=>""}],
+  ['hdr24', '-t: -H rmdup 1', {IN_PIPE=>""}, {OUT=>""}],
+
 
   # Test single line per group
   ['sl1', '-t" " -g 1 mean 2', {IN_PIPE=>$in_g4},
@@ -456,6 +618,11 @@ my @Tests =
      {OUT=>"A 1\nK 1\nP 1\n"}],
   ['cuq3', '-t" " --header-in -g 1 countunique 2', {IN_PIPE=>$in_cnt_uniq1},
      {OUT=>"A 2\nB 2\n"}],
+  # countunique, case-insensitive
+  ['cuq4', '-t" " -g 1 countunique 2', {IN_PIPE=>$in_cnt_uniq2},
+     {OUT=>"a 1\nA 3\n"}],
+  ['cuq5', '-i -t" " -g 1 countunique 2', {IN_PIPE=>$in_cnt_uniq2},
+     {OUT=>"a 2\n"}],
 
   # Test Tab vs White-space field separator
   ['tab1', "sum 2", {IN_PIPE=>$in_tab1}, {OUT=>"60\n"}],
@@ -472,6 +639,17 @@ my @Tests =
   # because the input is not sorted.
   ['sort3', '-g 2 unique 3', {IN_PIPE=>$in_sort1},
      {OUT=>"x\t1\nk\t2\nj\t3\nx\t5\nj\t4\ng\t6\n"}],
+  ['sort4', '-s -g 1 sum 2 -t "\'" ', {IN_PIPE=>$in_sort_quote1},
+     {OUT=>"A'4\nB'6\n"}],
+  ['sort5', '-s -g 1 sum 2 -t "\"" ', {IN_PIPE=>$in_sort_quote2},
+     {OUT=>"A\"4\nB\"6\n"}],
+
+#my $in_sort_quote1=<<"EOF";
+#A'1
+#B'2
+#A'3
+#B'4
+#EOF
 
 
   # Test Case-sensitivity, on sorted input (no 'sort' piping)
@@ -484,6 +662,8 @@ my @Tests =
      {OUT=>"a X,x\nA X,x\nb Y\nB Y\n"}],
   ['case4', '-t" " -i -g 1 unique 2', {IN_PIPE=>$in_case_sorted},
      {OUT=>"a X\nb Y\n"}],
+  ['case4.1', '-i -t" " -g 1 unique 2', {IN_PIPE=>$in_cnt_uniq2},
+     {OUT=>"a B,C\n"}],
 
   # Test Case-sensitivity, on non-sorted input (with 'sort' piping)
   # on both grouping and string operations.
@@ -500,9 +680,9 @@ my @Tests =
 
   # Test --help (but don't verify the output)
   ['help1', '--help',     {IN_PIPE=>""},  {OUT => ""},
-	  {OUT_SUBST=>'s/^.*//gms'}],
+      {OUT_SUBST=>'s/^.*//gms'}],
   ['ver1', '--version',     {IN_PIPE=>""},  {OUT => ""},
-	  {OUT_SUBST=>'s/^.*//gms'}],
+      {OUT_SUBST=>'s/^.*//gms'}],
 
   # Test output precision (number of digits) for numerical operations.
   # The current precision is 14 digits (hard-coded).
@@ -536,6 +716,121 @@ my @Tests =
             {OUT=>"A 2\nB 4\n"}],
   ['lst6',   '-t" " -g 1 last 2', {IN_PIPE=>$in_large_buffer2},
             {OUT=>$out_large_buffer_last}],
+
+  ## Test md5/sha1/sha256/sha512 operations.
+  ## NOTE: this just ensures the operations don't fail, and produces the
+  ##       expected length of output strings.
+  ##       validation of the calculation is tested in datamash-{md5,sha}.pl.
+  ['md5-1',   '-W md5 2',    {IN_PIPE=>$in_g1},
+      {OUT_SUBST=>'s/^[a-f0-9]{32}$//'}, {OUT=>"\n\n\n\n"}],
+  ['sha1-1',   '-W sha1 2',    {IN_PIPE=>$in_g1},
+      {OUT_SUBST=>'s/^[a-f0-9]{40}$//'}, {OUT=>"\n\n\n\n"}],
+  ['sha256-1',   '-W sha256 2',    {IN_PIPE=>$in_g1},
+      {OUT_SUBST=>'s/^[a-f0-9]{64}$//'}, {OUT=>"\n\n\n\n"}],
+  ['sha512-1',   '-W sha512 2',    {IN_PIPE=>$in_g1},
+      {OUT_SUBST=>'s/^[a-f0-9]{128}$//'}, {OUT=>"\n\n\n\n"}],
+
+  ## Test Base64
+  ['base64-1','-W base64 2', {IN_PIPE=>$in_g1}, {OUT=>$out_g1_base64}],
+  ['debase64-1','-W debase64 1', {IN_PIPE=>$out_g1_base64},
+    {OUT=>$out_g1_debase64}],
+  # Test invalid base64 input
+  ['debase64-2', '-W debase64 1', {IN_PIPE=>$in_g1}, {EXIT=>1},
+    {ERR=>"$prog: invalid base64 value in line 1 field 1: 'A'\n"}],
+
+  ## Mixing grouping,line,transpose/reverse operators should fail
+  ['mixop1', 'sum 1 md5 2', {EXIT=>1},
+    {ERR=>"$prog: conflicting operation found: expecting grouping operations," .
+            " but found line operation 'md5'\n"}],
+  ['mixop2', 'md5 1 sum 2', {EXIT=>1},
+    {ERR=>"$prog: conflicting operation found: expecting line operations," .
+            " but found grouping operation 'sum'\n"}],
+  ['mixop3', 'md5 1 transpose 2', {EXIT=>1},
+    {ERR=>"$prog: conflicting operation found: expecting line operations," .
+            " but found transpose operation 'transpose'\n"}],
+
+  # Test large (and increasing) number of fields
+  ['wide1', '-t" " -g 1 countunique 2', {IN_PIPE=>$in_wide1}, {OUT=>"A 1\n"}],
+
+  # Test rmdup operation
+  ['rmdp1', '-t: rmdup 1', {IN_PIPE=>""}, {OUT=>""}],
+  ['rmdp2', '-t: rmdup 1', {IN_PIPE=>$in_dup1},
+    {OUT=>"X:Y\n1:a\n2:b\n3:a\n"}],
+  ['rmdp3', '-t: rmdup 2', {IN_PIPE=>$in_dup1},
+    {OUT=>"X:Y\n1:a\n2:b\n2:c\n"}],
+  # in+out header: same as no header at all (assuming the header is unique)
+  ['rmdp4', '-t: -H rmdup 1', {IN_PIPE=>$in_dup1},
+    {OUT=>"X:Y\n1:a\n2:b\n3:a\n"}],
+  ['rmdp5', '-t: --header-in rmdup 1', {IN_PIPE=>$in_dup1},
+    {OUT=>"1:a\n2:b\n3:a\n"}],
+  ['rmdp6', '-t: rmdup 4', {IN_PIPE=>$in_dup1}, {EXIT=>1},
+      {ERR=>"$prog: invalid input: field 4 requested, " .
+            "line 1 has only 2 fields\n"}],
+  ## rmdup with named columns
+  ['rmdp7', '-t: -H rmdup X', {IN_PIPE=>$in_dup1},
+    {OUT=>"X:Y\n1:a\n2:b\n3:a\n"}],
+  ['rmdp8', '-t: rmdup X', {IN_PIPE=>$in_dup1}, {EXIT=>1},
+     {ERR=>"$prog: -H or --header-in must be used with named columns\n"}],
+
+  # Test noop operation
+  ['noop1', 'noop', {IN_PIPE=>""}, {OUT=>""}],
+  ['noop2', 'noop', {IN_PIPE=>$in_dup1}, {OUT=>""}],
+  ['noop3', '--full noop', {IN_PIPE=>$in_dup1}, {OUT=>$in_dup1}],
+
+
+  # Test named columns - for operations
+  ## with empty input, invalid column name is not an error
+  ['nc2',   '-H sum foo', {IN_PIPE=>""}, {OUT=>""}],
+  ['nc3',   '--header-in sum foo', {IN_PIPE=>""}, {OUT=>""}],
+  ## with named columns, -H or --header-in are required.
+  ['nc1',   'sum foo', {IN_PIPE=>""}, {EXIT=>1},
+     {ERR=>"$prog: -H or --header-in must be used with named columns\n"}],
+  ['nc4',   'sum foo', {IN_PIPE=>$in_hdr1}, {EXIT=>1},
+     {ERR=>"$prog: -H or --header-in must be used with named columns\n"}],
+  ## Invalid column name
+  ['nc5',   '--header-in sum foo', {IN_PIPE=>$in_hdr1}, {EXIT=>1},
+     {ERR=>"$prog: column name 'foo' not found in input file\n"}],
+  ['nc6',   '-H sum foo', {IN_PIPE=>$in_hdr1}, {EXIT=>1},
+     {ERR=>"$prog: column name 'foo' not found in input file\n"}],
+  ## Valid column names, and identical column number
+  ['nc7',   '-t" " -H sum y sum 2', {IN_PIPE=>$in_hdr1},
+     {OUT=>"sum(y) sum(y)\n52 52\n"}],
+  ## Valid column names + sorting
+  ['nc8',   '-s -g 1 -t" " -H sum y', {IN_PIPE=>$in_hdr1},
+     {OUT=>"GroupBy(x) sum(y)\nA 14\nB 18\nC 20\n"}],
+
+  # Test named columns - for grouping
+  ## with empty input, invalid column name is not an error
+  ['ng1',   '-H -g foo sum 1', {IN_PIPE=>""}, {OUT=>""}],
+  ['ng2',   '--header-in -g foo sum foo', {IN_PIPE=>""}, {OUT=>""}],
+  ## with named columns, -H or --header-in are required
+  ['ng3',   '-g foo count 1', {IN_PIPE=>""}, {EXIT=>1},
+     {ERR=>"$prog: -H or --header-in must be used with named columns\n"}],
+  ['ng4',   '-g foo count 1', {IN_PIPE=>$in_hdr1}, {EXIT=>1},
+     {ERR=>"$prog: -H or --header-in must be used with named columns\n"}],
+  ## Invalid column name
+  ['ng5',   '--header-in -g foo sum 1', {IN_PIPE=>$in_hdr1}, {EXIT=>1},
+     {ERR=>"$prog: column name 'foo' not found in input file\n"}],
+  ['ng6',   '-H -g foo sum 1', {IN_PIPE=>$in_hdr1}, {EXIT=>1},
+     {ERR=>"$prog: column name 'foo' not found in input file\n"}],
+  ## Valid column names in grouping
+  ['ng7',   '-t" " -H -g x sum 2', {IN_PIPE=>$in_hdr1},
+     {OUT=>"GroupBy(x) sum(y)\nA 14\nB 18\nC 20\n"}],
+  ## group by name+number
+  ['ng8',   '-t":" -H -g x,3 sum 2', {IN_PIPE=>$in_hdr2},
+     {OUT=>"GroupBy(x):GroupBy(z):sum(y)\n" .
+	   "A:W:15\nA:X:24\nB:Y:17\nB:Z:19\nC:Z:23\n"}],
+  ## group by name+name
+  ['ng9',   '-t":" -H -g x,z sum 2', {IN_PIPE=>$in_hdr2},
+     {OUT=>"GroupBy(x):GroupBy(z):sum(y)\n" .
+	   "A:W:15\nA:X:24\nB:Y:17\nB:Z:19\nC:Z:23\n"}],
+  ## group by number+name
+  ['ng10',  '-t":" -H -g 1,z sum 2', {IN_PIPE=>$in_hdr2},
+     {OUT=>"GroupBy(x):GroupBy(z):sum(y)\n" .
+	   "A:W:15\nA:X:24\nB:Y:17\nB:Z:19\nC:Z:23\n"}],
+  ## group by name + sorting
+  ['ng11',   '-s -t" " -H -g x sum 2', {IN_PIPE=>$in_hdr1},
+     {OUT=>"GroupBy(x) sum(y)\nA 14\nB 18\nC 20\n"}],
 );
 
 if ($have_stable_sort) {
@@ -544,7 +839,7 @@ if ($have_stable_sort) {
     ['lst7',   '-t" " --sort -g 1 last 3',  {IN_PIPE=>$in_case_unsorted},
        {OUT=>"A 5\nB 6\na 3\nb 4\n"}],
     ['lst8',   '-t" " --sort -i -g 1 last 3',  {IN_PIPE=>$in_case_unsorted},
-       {OUT=>"a 5\nb 6\n"}],
+       {OUT=>"A 5\nB 6\n"}],
     # First with sort, test the 'stable' sort
     ['fst7',   '-t" " --sort -g 1 first 3',  {IN_PIPE=>$in_case_unsorted},
        {OUT=>"A 2\nB 6\na 1\nb 4\n"}],
