@@ -4,7 +4,7 @@
   Tests for 'transpose' and 'reverse' operation modes.
 
 
-   Copyright (C) 2013,2014 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2015 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -128,6 +128,13 @@ my $out3_filler_tr = perl_transpose ("\t", "xxx", $in3 ) ;
 my $out4_tr = perl_transpose ("\t", "N/A", $in4 ) ;
 my $out5_tr = perl_transpose ("\t", "N/A", $in5 ) ;
 
+my $in_hdr1=<<'EOF';
+X:Y
+1:a
+2:b
+EOF
+
+
 my @Tests =
 (
     # Simple transpose and reverse
@@ -176,6 +183,24 @@ my @Tests =
       {ERR=>"$prog: extra operands after 'transpose'\n"}],
     ['rev9', 'reverse aaa', {IN_PIPE=>''}, {EXIT=>1},
       {ERR=>"$prog: extra operands after 'reverse'\n"}],
+
+    # empty input
+    ['tr10',  'transpose', {IN_PIPE=>""}, {OUT=>""}],
+    ['rev10', 'reverse',   {IN_PIPE=>""}, {OUT=>""}],
+
+    # Reverse with header combinations
+    ['rev-hdr1','-H reverse', {IN_PIPE=>""}, {OUT=>""}],
+    ['rev-hdr2','--header-in reverse', {IN_PIPE=>""}, {OUT=>""}],
+    ['rev-hdr3','-t: reverse', {IN_PIPE=>$in_hdr1},
+       {OUT=>"Y:X\na:1\nb:2\n"}],
+    ['rev-hdr4','-t: -H reverse', {IN_PIPE=>$in_hdr1},
+       {OUT=>"Y:X\na:1\nb:2\n"}],
+    # first line is header line, discard it (there's no --header-out).
+    ['rev-hdr5','-t: --header-in reverse', {IN_PIPE=>$in_hdr1},
+       {OUT=>"a:1\nb:2\n"}],
+    # Generate a new header, assuming the first line is a NOT header line.
+    ['rev-hdr6','-t: --header-out reverse', {IN_PIPE=>$in_hdr1},
+       {OUT=>"field-2:field-1\nY:X\na:1\nb:2\n"}],
 );
 
 my $save_temps = $ENV{SAVE_TEMPS};
