@@ -2,6 +2,9 @@
 
 # Copyright (C) 2009-2014 Free Software Foundation, Inc.
 
+# Modifications for GNU Datamash are
+# Copyright (C) 2014-2015 Assaf Gordon <assafgordon@gmail.com>
+
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -201,6 +204,8 @@ else
       *x*) opts_=-x ;;
       *) opts_= ;;
     esac
+    re_shell=$re_shell_
+    export re_shell
     exec "$re_shell_" $opts_ "$0" --no-reexec "$@"
     echo "$ME_: exec failed" 1>&2
     exit 127
